@@ -3,13 +3,13 @@ use strict;
 use warnings;
 no warnings 'uninitialized';
 use Exporter 'import';
-our @EXPORT_OK = qw(new head header footer popup page pages form listview collapsible collapsible_set navbar button controlgroup input select checkbox radio textarea);
+our @EXPORT_OK = qw(new head header footer table panel popup page pages form listview collapsible collapsible_set navbar button controlgroup input rangeslider select checkbox radio textarea);
 
 use Clone qw(clone);
 use HTML::Entities qw(encode_entities);
 
-our $VERSION = 0.01;
-# 36.3
+our $VERSION = 0.02;
+# 53.4
 
 sub new {
 	my ($class, %args) = (@_);
@@ -24,15 +24,15 @@ sub new {
 		'apple-touch-icon-72' => '', # path to apple web app icon image (72x72 pixels)
 		'apple-touch-icon-114' => '', # path to apple web app icon image (114x114 pixels)
 		'apple-touch-startup-image' => '', # path to apple web app startup image
-		'jquery-mobile-css' => 'http://code.jquery.com/mobile/1.2.0/jquery.mobile-1.2.0.min.css',
-		'jquery-mobile-js' => 'http://code.jquery.com/mobile/1.2.0/jquery.mobile-1.2.0.min.js',
-		'jquery' => 'http://code.jquery.com/jquery-1.8.2.min.js',
+		'jquery-mobile-css' => 'http://code.jquery.com/mobile/1.3.2/jquery.mobile-1.3.2.min.css',
+		'jquery-mobile-js' => 'http://code.jquery.com/mobile/1.3.2/jquery.mobile-1.3.2.min.js',
+		'jquery' => 'http://code.jquery.com/jquery-1.9.1.min.js',
 		'app-css' => [], # global application CSS files
 		'app-js' => [], # global application JS files
 		'app-inline-css' => '      span.invalid{color:#F00000;line-height: 1.5;}', # inline CSS code
 		'app-inline-js' => '', # inline JS code
 		'app-title' => '', # <title> in <head>
-		# a list of default allowed HTML and data-* attributes for UI components (Reference: http://jquerymobile.com/test/docs/api/data-attributes.html)
+		# a list of default allowed HTML and data-* attributes for UI components (Reference: http://api.jquerymobile.com/data-attribute/)
 		'header-footer-html-attribute' => ['id', 'class'],
 		'header-footer-data-attribute' => ['id', 'fullscreen', 'position', 'theme'],
 		'navbar-html-attribute' => ['id', 'class'],
@@ -41,11 +41,17 @@ sub new {
 		'navbar-item-data-attribute' => ['ajax', 'icon', 'iconpos', 'iconshadow','prefetch', 'theme'],
 		'page-html-attribute' => ['id', 'class'],
 		# combine data-attributes for page and dialog
-		'page-data-attribute' => ['add-back-btn', 'back-btn-text', 'back-btn-theme', 'close-btn-text', 'corners', 'dom-cache', 'enhance', 'overlay-theme', 'role', 'shadow','theme', 'title', 'tolerance', 'url'],
+		'page-data-attribute' => ['add-back-btn', 'back-btn-text', 'back-btn-theme', 'close-btn', 'close-btn-text', 'corners', 'dom-cache', 'enhance', 'overlay-theme', 'role', 'shadow','theme', 'title', 'tolerance', 'url'],
+		'table-html-attribute' => ['id', 'class'],
+		'table-data-attribute' => ['mode'],
+		'table-head-html-attribute' => ['id', 'class'],
+		'table-head-data-attribute' => ['priority'],
+		'panel-html-attribute' => ['id', 'class'],
+		'panel-data-attribute' => ['corners', 'overlay-theme', 'shadow', 'theme', 'tolerance', 'position-to', 'rel', 'role', 'transition'],
 		'popup-html-attribute' => ['id', 'class'],
-		'popup-data-attribute' => ['corners', 'overlay-theme', 'shadow', 'theme', 'tolerance', 'position-to', 'rel', 'role', 'transition'],
+		'popup-data-attribute' => ['animate', 'dismissible', 'display', 'position', 'position-fixed', 'swipe-close', 'role', 'theme'],
 		'listview-html-attribute' => ['id', 'class'],
-		'listview-data-attribute' => ['autodividers', 'count-theme', 'divider-theme', 'enhance', 'filter', 'filter-placeholder', 'filter-theme', 'filtertext', 'header-theme', 'inset', 'split-icon', 'split-theme', 'theme'],
+		'listview-data-attribute' => ['autodividers', 'count-theme', 'divider-theme', 'enhance', 'filter', 'filter-placeholder', 'filter-reveal', 'filter-theme', 'filtertext', 'header-theme', 'inset', 'split-icon', 'split-theme', 'theme'],
 		'listview-item-html-attribute' => ['id', 'class'],
 		'listview-item-data-attribute' => ['ajax', 'mini', 'rel', 'theme', 'transition'],
 		'collapsible-html-attribute' => ['id', 'class'],
@@ -60,13 +66,15 @@ sub new {
 		'form-html-attribute' => ['id', 'action', 'class', 'enctype', 'method'],
 		'form-data-attribute' => ['enhance', 'theme', 'ajax'],
 		'input-html-attribute' => ['id', 'class', 'disabled', 'max', 'maxlength', 'min', 'name', 'pattern', 'placeholder', 'readonly', 'required', 'size', 'type', 'value', 'accept', 'capture'],
-		'input-data-attribute' => ['corners', 'highlight', 'icon', 'iconpos', 'iconshadow', 'inline', 'mini', 'shadow', 'theme', 'track-theme'],
+		'input-data-attribute' => ['clear-btn', 'clear-btn-text', 'corners', 'highlight', 'icon', 'iconpos', 'iconshadow', 'inline', 'mini', 'shadow', 'theme', 'track-theme'],
 		'textarea-html-attribute' => ['id', 'name', 'class', 'rows', 'cols', 'readonly', 'disabled', 'title', 'required', 'placeholder', 'title', 'pattern'],
-		'textarea-data-attribute' => ['mini', 'theme'],
-		'select-html-attribute' => ['id', 'class', 'size', 'maxlength', 'readonly', 'disabled', 'title', 'required', 'placeholder', 'title', 'pattern'],
-		'select-data-attribute' => ['icon', 'iconpos', 'inline', 'mini', 'native-menu', 'overlay-theme', 'placeholder', 'theme'],
+		'textarea-data-attribute' => ['clear-btn', 'clear-btn-text', 'mini', 'theme'],
+		'select-html-attribute' => ['id', 'class', 'size', 'maxlength', 'readonly', 'disabled', 'title', 'required', 'placeholder', 'title', 'pattern', 'multiple'],
+		'select-data-attribute' => ['icon', 'iconpos', 'inline', 'mini', 'native-menu', 'overlay-theme', 'theme', 'role'],
 		'radio-checkbox-html-attribute' => ['id', 'class', 'readonly', 'disabled', 'title', 'required', 'placeholder', 'title', 'pattern', 'value'],
 		'radio-checkbox-data-attribute' => ['mini', 'theme'],
+		'rangeslider-html-attribute' => ['id', 'name', 'class'],
+		'rangeslider-data-attribute' => ['highlight', 'mini', 'theme', 'track-theme'],
 		'label' => sub {
 			my $args = shift;
 			return '<strong>' . $args->{label} . '</strong>' if $args->{required};
@@ -202,6 +210,54 @@ sub navbar {
 	return $navbar;
 }
 
+sub panel {
+	my ($self, %args) = @_;
+
+	$args{content} ||= '          <p>Panel Content</p>';
+	$args{role} = 'panel';
+
+	my $attributes = _html_attribute('', $self->{config}->{'panel-html-attribute'}, \%args);
+	$attributes = _data_attribute($attributes, $self->{config}->{'panel-data-attribute'}, \%args);
+
+	my $panel = '        <div' . $attributes . '>' . "\n";
+	$panel .= $args{content} . "\n";
+	$panel .= '        </div><!-- /panel -->' . "\n";
+
+	return $panel;
+}
+
+sub table {
+	my ($self, %args) = @_;
+
+	unless ($args{content}) {
+
+		$args{content} =  '          <thead>' . "\n" . '            <tr>' . "\n";
+		foreach my $header (@{$args{headers}}) {
+
+			my $head_attributes = '';
+			if ($args{th} && exists $args{th}->{$header}) {
+				$head_attributes = _html_attribute($head_attributes, $self->{config}->{'table-head-html-attribute'}, $args{th}->{$header});
+				$head_attributes = _data_attribute($head_attributes, $self->{config}->{'table-head-data-attribute'}, $args{th}->{$header});
+			}
+
+			$args{content} .= '              <th' . $head_attributes . '>' . $header . '</th>' . "\n";
+		}
+		$args{content} .=  '            </tr>' . "\n" . '          </thead>' . "\n";
+
+		$args{content} .=  '          <tbody>'. "\n";
+		foreach my $row (@{$args{rows}}) {
+			$args{content} .=  '            <tr>' . "\n" . join("\n", map ({'              <td>' . $_ . '</td>'} @{$row})) . "\n" . '            </tr>' . "\n";
+		}
+		$args{content} .=  '          </tbody>';
+	}
+
+	my $attributes = _html_attribute('', $self->{config}->{'table-html-attribute'}, \%args);
+	$attributes = _data_attribute($attributes, $self->{config}->{'table-data-attribute'}, \%args);
+
+	my $table = '        <table data-role="table"' . $attributes . '>' . "\n" . $args{content} . "\n" . '        </table>' . "\n";
+	return $table;
+}
+
 sub popup {
 	my ($self, %args) = @_;
 
@@ -229,6 +285,7 @@ sub page {
 
 	my $page = '    <div' . $attributes . '>' . "\n";
 	$page .= $self->header(%{$args{header}}) if $args{header};
+	$page .= $self->panel(%{$args{panel}}) if $args{panel};
 	$page .= '      <div data-role="content">' . "\n" . $args{content} . "\n" . '      </div><!-- /content -->' . "\n";
 	$page .= $args{after} if $args{after};
 	$page .= $self->footer(%{$args{footer}}) if $args{footer};
@@ -273,30 +330,30 @@ sub collapsible_set {
 	my $attributes = _html_attribute('', $self->{config}->{'collapsible-set-html-attribute'}, \%args);
 	$attributes = _data_attribute($attributes, $self->{config}->{'collapsible-set-data-attribute'}, \%args);
 
-	my $collapsible_set = '        <div data-role="collapsible-set"' . $attributes . '>' . "\n" . $args{content} . "\n" . '        </div>';
+	my $collapsible_set = '        <div data-role="collapsible-set"' . $attributes . '>' . "\n" . $args{content} . "\n" . '        </div>' . "\n";
 	return $collapsible_set;
 }
 
 sub collapsible {
 	my ($self, %args) = @_;
 
-	if ($args{title} && $args{listview}) {
+	if ($args{listview}) {
 		if ($args{active} && ! exists $args{listview}->{active}) {
 			$args{listview}->{active} = $args{active};
 		}
 
-		$args{content} = '          <h1>' . $args{title} . '</h1>' . "\n" .  $self->listview(%{$args{listview}});
-
+		$args{title} ||= 'Title';
+		$args{content} = '          <h1>' . $args{title} . '</h1>' . "\n" . $self->listview(%{$args{listview}});
 		$args{collapsed} = 'false' if ! exists $args{collapsed} && $args{content} =~ /ui-btn-active/;
 	}
 	else {
-		$args{content} ||= '            <p>Collapsible Content</p>';
+		$args{content} ||= '            <h1>Collapsible Title</h1><p>Collapsible Content</p>';
 	}
 
 	my $attributes = _html_attribute('', $self->{config}->{'collapsible-html-attribute'}, \%args);
 	$attributes = _data_attribute($attributes, $self->{config}->{'collapsible-data-attribute'}, \%args);
 
-	my $collapsible = '          <div data-role="collapsible"' . $attributes . '>' . "\n" . $args{content} . "\n" . '          </div>';
+	my $collapsible = '          <div data-role="collapsible"' . $attributes . '>' . "\n" . $args{content} . "\n" . '          </div>' . "\n";
 	return $collapsible;
 }
 
@@ -328,7 +385,7 @@ sub listview {
 
 	my $list = '        <' . $list_tag . ' data-role="listview"' . $attributes . '>' . "\n";
 	
-	my $divider = '';
+	my $divider = ''; 
 
 	foreach my $item (@{$args{items}}) {
 
@@ -414,9 +471,11 @@ sub controlgroup {
 	my $attributes = _html_attribute('', $self->{config}->{'controlgroup-html-attribute'}, \%args);
 	$attributes = _data_attribute($attributes, $self->{config}->{'controlgroup-data-attribute'}, \%args);
 
-	my $controlgroup = '<div data-role="controlgroup"'. $attributes . '>' . "\n";
-	$controlgroup .= $args{content} . "\n";
-	$controlgroup .= '</div><!-- /controlgroup -->';
+	my $element = $args{fieldset} ? 'fieldset' : 'div';
+
+	my $controlgroup = '          <' . $element . ' data-role="controlgroup"'. $attributes . '>' . "\n";
+	$controlgroup .= '          ' . $args{content} . "\n";
+	$controlgroup .= '          </' . $element . '><!-- /controlgroup -->' . "\n";
 	return $controlgroup;
 }
 
@@ -451,7 +510,7 @@ sub form {
 
 	if ($args{fields}) {
 		foreach my $field (@{$args{fields}}) {
-			if ($field->{type} && $field->{type} =~ /^(select|radio|checkbox|textarea)$/) {
+			if ($field->{type} && $field->{type} =~ /^(select|radio|checkbox|textarea|rangeslider)$/) {
 				my $type = delete $field->{type};
 				$content .= $self->$type(%{$field});
 			}
@@ -492,9 +551,29 @@ sub form {
 	return $form;
 }
 
-sub input {
+sub rangeslider {
 	my ($self, %args) = @_;
 
+	my $attributes = _html_attribute('', $self->{config}->{'rangeslider-html-attribute'}, \%args);
+	$attributes = _data_attribute($attributes, $self->{config}->{'rangeslider-data-attribute'}, \%args);
+
+	$args{from}->{type} = 'range';
+	$args{to}->{type} = 'range';
+	my $from = '              ' . $self->_input(%{$args{from}}) . "\n";
+	my $to = '              ' . $self->_input(%{$args{to}});
+
+	$args{container_role} ||= 'fieldcontain';
+	my $invalid = $args{invalid} ? $self->{config}->{invalid}->(\%args) : '';
+
+	my $rangeslider = '          <div data-role="' . $args{container_role} . '">' . "\n" . '            <div data-role="rangeslider"' . $attributes . '>' . "\n";
+	$rangeslider .= $from . $to . $invalid . "\n";
+	$rangeslider .= "            </div>\n          </div>\n";
+
+	return $rangeslider;
+}
+
+sub _input {
+	my ($self, %args) = @_;
 	$args{type} ||= 'text';
 	$args{id} ||= $args{name};
 	$args{label} ||= _label($args{name});
@@ -505,9 +584,18 @@ sub input {
 	$attributes = _data_attribute($attributes, $self->{config}->{'input-data-attribute'}, \%args);
 
 	return '          <input' . $attributes . ' />' . "\n" if $args{type} eq 'hidden';
+	return '<label for="' . $args{id} . '">' . $self->{config}->{label}->(\%args) .  ':</label><input' . $attributes . ' />';
+}
 
+sub input {
+	my ($self, %args) = @_;
+
+	my $input = $self->_input(%args);
+	return $input if $args{type} eq 'hidden';
+
+	$args{container_role} ||= 'fieldcontain';
 	my $invalid = $args{invalid} ? $self->{config}->{invalid}->(\%args) : '';
-	return '          <div data-role="fieldcontain"><label for="' . $args{id} . '">' . $self->{config}->{label}->(\%args) .  ':</label><input' . $attributes . ' />' . $invalid . '</div>' . "\n";
+	return '          <div data-role="' . $args{container_role} . '">' . $input . $invalid . '</div>' . "\n";
 }
 
 sub textarea {
@@ -523,8 +611,9 @@ sub textarea {
 	my $attributes = _html_attribute('', $self->{config}->{'textarea-html-attribute'}, \%args);
 	$attributes = _data_attribute($attributes, $self->{config}->{'textarea-data-attribute'}, \%args);
 
+	$args{container_role} ||= 'fieldcontain';
 	my $invalid = $args{invalid} ? $self->{config}->{invalid}->(\%args) : '';
-	return '          <div data-role="fieldcontain"><label for="' . $args{id} . '">' . $self->{config}->{label}->(\%args) .  ':</label><textarea' . $attributes . '>' . $args{value} . '</textarea>' . $invalid . '</div>'. "\n";
+	return '          <div data-role="' . $args{container_role} . '"><label for="' . $args{id} . '">' . $self->{config}->{label}->(\%args) .  ':</label><textarea' . $attributes . '>' . $args{value} . '</textarea>' . $invalid . '</div>' . "\n";
 }
 
 sub select {
@@ -533,30 +622,90 @@ sub select {
 	$args{id} ||= $args{name};
 	$args{label} ||= _label($args{name});
 
+	if ($args{multiple}) {
+		$args{'native-menu'} ||= 'false';
+	}
+
 	my $attributes = _html_attribute('', $self->{config}->{'select-html-attribute'}, \%args);
 	$attributes = _data_attribute($attributes, $self->{config}->{'select-data-attribute'}, \%args);
 
 	my $options = '';
+	my $placeholder_text = $args{placeholder_text};
 
 	if (ref $args{options} eq 'HASH') {
-		foreach my $key (keys %{$args{options}}) {
+
+		my @keys;
+		my $sort_options = $args{sort_options}; 
+		if ($sort_options) {
+			if ($sort_options eq 'key') {
+				@keys = sort keys %{$args{options}};
+			}
+			else {
+				@keys = sort {$args{options}->{$a} cmp $args{options}->{$b}} keys %{$args{options}};
+			}
+		}
+		else {
+			@keys = keys %{$args{options}};
+		}
+
+		foreach my $key (@keys) {
 			my $selected = '';
-			$selected = 'selected="selected"' if $key eq $args{value};
+
+			if (defined $args{value}) {
+
+				if (ref $args{value} eq 'HASH') {
+					foreach my $value_key (keys %{$args{value}}) {
+						if ($key eq $value_key) {
+							$selected = 'selected="selected"';
+							last;
+						}
+					}
+				}
+				elsif ($key eq $args{value}) {
+					$selected = 'selected="selected"';
+				}
+			}
 
 			$options .= '<option value="' . $key . '" ' . $selected . '>' . encode_entities($args{options}->{$key}) . '</option>';
 		}
+
+		if (defined $placeholder_text) {
+			$options = '<option value="">' . encode_entities($placeholder_text) . '</option>' . $options;
+		}
 	}
-	else {
+	elsif (ref $args{options} eq 'ARRAY') {
 		foreach my $option (@{$args{options}}) {
 			my $selected = '';
-			$selected = 'selected="selected"' if $option eq $args{value};
+
+			if (defined $args{value}) {
+
+				if (ref $args{value} eq 'ARRAY') {
+					foreach my $element (@{$args{value}}) {
+						if ($option eq $element) {
+							$selected = 'selected="selected"';
+							last;
+						}
+					}
+				}
+				elsif ($option eq $args{value}) {
+					$selected = 'selected="selected"';
+				}
+			}
 
 			$options .= '<option value="' . $option . '" ' . $selected . '>' . encode_entities($option) . '</option>';
 		}
+
+		if (defined $placeholder_text) {
+			$options = '<option value="">' . encode_entities($placeholder_text) . '</option>' . $options;
+		}
+	}
+	else {
+		$options = $args{options};
 	}
 
+	$args{container_role} ||= 'fieldcontain';
 	my $invalid = $args{invalid} ? $self->{config}->{invalid}->(\%args) : '';
-	return '          <div data-role="fieldcontain"><label for="' . $args{id} . '">' . $self->{config}->{label}->(\%args) .  ':</label><select name="' . $args{name} . '"' . $attributes . '>' . $options . '</select>' . $invalid . '</div>' . "\n";
+	return '          <div data-role="' . $args{container_role} . '"><label for="' . $args{id} . '">' . $self->{config}->{label}->(\%args) .  ':</label><select name="' . $args{name} . '"' . $attributes . '>' . $options . '</select>' . $invalid . '</div>' . "\n";
 }
 
 sub radio {
@@ -579,37 +728,88 @@ sub _radio_checkbox {
 	$args{id} ||= $args{name};
 	$args{label} ||= _label($args{name});
 
-	$args{value} = encode_entities($args{value});
-
 	my $data_attributes = _data_attribute('', $self->{config}->{'radio-checkbox-data-attribute'}, \%args);
 	my $cloned_args = clone(\%args);
 	my $options = '';
 
 	if (ref $args{options} eq 'HASH') {
-		foreach my $key (keys %{$args{options}}) {
-			$cloned_args->{id} = $args{name} . '-' . _urlise($key);
+
+		my @keys;
+		my $sort_options = $args{sort_options}; 
+		if ($sort_options) {
+			if ($sort_options eq 'key') {
+				@keys = sort keys %{$args{options}};
+			}
+			else {
+				@keys = sort {$args{options}->{$a} cmp $args{options}->{$b}} keys %{$args{options}};
+			}
+		}
+		else {
+			@keys = keys %{$args{options}};
+		}
+
+		foreach my $key (@keys) {
+			$cloned_args->{id} = $args{name} . '-' . _id($key);
 			$cloned_args->{value} = $key;
 			my $html_attributes = _html_attribute('', $self->{config}->{'radio-checkbox-html-attribute'}, $cloned_args);
 
 			my $checked = '';
-			$checked = ' checked="checked"' if defined $args{value} && $key eq $args{value};
+
+			if (defined $args{value}) {
+
+				if (ref $args{value} eq 'HASH') {
+					foreach my $value_key (keys %{$args{value}}) {
+						if ($key eq $value_key) {
+							$checked = ' checked="checked"';
+							last;
+						}
+					}
+				}
+				elsif ($key eq $args{value}) {
+					$checked = ' checked="checked"';
+				}
+			}
+
 			$options .= '<input type="' . $args{type} . '" name="' . $args{name} . '"' . $html_attributes . $data_attributes . $checked . ' /><label for="' . $cloned_args->{id} . '">' . $args{options}->{$key} . '</label>';
 		}
 	}
-	else {
-		foreach my $option (@{$args{options}}) {
-			$cloned_args->{id} = $args{name} . '-' . _urlise($option);
-			$cloned_args->{value} = $option;
+	elsif (ref $args{options} eq 'ARRAY') {
+		foreach my $key (@{$args{options}}) {
+			$cloned_args->{id} = $args{name} . '-' . _id($key);
+			$cloned_args->{value} = $key;
 			my $html_attributes = _html_attribute('', $self->{config}->{'radio-checkbox-html-attribute'}, $cloned_args);
 
 			my $checked = '';
-			$checked = ' checked="checked"' if $option eq $args{value};
-			$options .= '<input type="' . $args{type} . '" name="' . $args{name} . '"' . $html_attributes . $data_attributes . $checked . ' /><label for="' . $cloned_args->{id} . '">' . $option . '</label>';
+
+			if (defined $args{value}) {
+
+				if (ref $args{value} eq 'ARRAY') {
+					foreach my $element (@{$args{value}}) {
+						if ($key eq $element) {
+							$checked = ' checked="checked"';
+							last;
+						}
+					}
+				}
+				elsif ($key eq $args{value}) {
+					$checked = ' checked="checked"';
+				}
+			}
+
+			$options .= '<input type="' . $args{type} . '" name="' . $args{name} . '"' . $html_attributes . $data_attributes . $checked . ' /><label for="' . $cloned_args->{id} . '">' . $key . '</label>';
 		}
 	}
+	else {
+		$options = $args{options};
+	}
 
 	my $invalid = $args{invalid} ? $self->{config}->{invalid}->(\%args) : '';
-	return '          <div data-role="fieldcontain"><fieldset data-role="controlgroup"><legend>' . $self->{config}->{label}->(\%args) .  ':</legend>' . $options . '</fieldset>' . $invalid . '</div>' . "\n";
+	
+	my $controlgroup = clone ($args{controlgroup});
+	$controlgroup->{fieldset} = 1;
+	$controlgroup->{content} ||= '  <legend>' . $self->{config}->{label}->(\%args) .  ':</legend>' . $options . $invalid;
+
+	return $self->controlgroup(%{$controlgroup});
 }
 
 sub _header_footer_attribute {
@@ -672,10 +872,11 @@ sub _label {
 	return $string;
 }
 
-sub _urlise {
+sub _id {
 	my $text = shift;
 	$text =~ s/&/and/g;
-	$text =~ s/[^0-9A-Za-z]//g;
+	$text =~ s/\//-/g;
+	$text =~ s/[^0-9A-Za-z\-_.:]//g;
 	return lc($text);
 }
 
@@ -745,9 +946,9 @@ Here is a list of optional parameters when instantiating a JQuery::Mobile object
       'apple-touch-icon-72' => '', # path to apple web app icon image (72x72 pixels)
       'apple-touch-icon-114' => '', # path to apple web app icon image (114x114 pixels)
       'apple-touch-startup-image' => '', # path to apple web app startup image
-      'jquery-mobile-css' => 'http://code.jquery.com/mobile/1.2.0/jquery.mobile-1.2.0.min.css',
-      'jquery-mobile-js' => 'http://code.jquery.com/mobile/1.2.0/jquery.mobile-1.2.0.min.js',
-      'jquery' => 'http://code.jquery.com/jquery-1.8.2.min.js',
+      'jquery-mobile-css' => 'http://code.jquery.com/mobile/1.3.2/jquery.mobile-1.3.2.min.css',
+      'jquery-mobile-js' => 'http://code.jquery.com/mobile/1.3.2/jquery.mobile-1.3.2.min.js',
+      'jquery' => 'http://code.jquery.com/jquery-1.9.1.min.js',
       'app-css' => [], # global application CSS files
       'app-js' => [], # global application JS files
       'app-inline-css' => '      span.invalid{color:#F00000;line-height: 1.5;}', # inline CSS code
@@ -756,7 +957,7 @@ Here is a list of optional parameters when instantiating a JQuery::Mobile object
      }
   );
 
-The allowed HTML and data-* attributes for each UI component can be customised. By default, HTML attributes are very stict to ensure a clean markup. For data-* attributes, see reference: http://jquerymobile.com/test/docs/api/data-attributes.html.
+The allowed HTML and data-* attributes for each UI component can be customised. By default, HTML attributes are very strict to ensure a clean markup. For data-* attributes, see reference: http://api.jquerymobile.com/data-attribute/.
 
   # default values are shown
   my $jquery_mobile = JQuery::Mobile->new(
@@ -769,7 +970,7 @@ The allowed HTML and data-* attributes for each UI component can be customised.
       'navbar-item-data-attribute' => ['ajax', 'icon', 'iconpos', 'iconshadow','prefetch', 'theme'],
       'page-html-attribute' => ['id', 'class'],
       # combine data-attributes for page and dialog
-      'page-data-attribute' => ['add-back-btn', 'back-btn-text', 'back-btn-theme', 'close-btn-text', 'corners', 'dom-cache', 'enhance', 'overlay-theme', 'role', 'shadow','theme', 'title', 'tolerance', 'url'],
+      'page-data-attribute' => ['add-back-btn', 'back-btn-text', 'back-btn-theme', 'close-btn', 'close-btn-text', 'corners', 'dom-cache', 'enhance', 'overlay-theme', 'role', 'shadow','theme', 'title', 'tolerance', 'url'],
       'popup-html-attribute' => ['id', 'class'],
       'popup-data-attribute' => ['corners', 'overlay-theme', 'shadow', 'theme', 'tolerance', 'position-to', 'rel', 'role', 'transition'],
       'listview-html-attribute' => ['id', 'class'],
@@ -788,9 +989,9 @@ The allowed HTML and data-* attributes for each UI component can be customised.
       'form-html-attribute' => ['id', 'action', 'class', 'enctype', 'method'],
       'form-data-attribute' => ['enhance', 'theme', 'ajax'],
       'input-html-attribute' => ['id', 'class', 'disabled', 'max', 'maxlength', 'min', 'name', 'pattern', 'placeholder', 'readonly', 'required', 'size', 'type', 'value', 'accept', 'capture'],
-      'input-data-attribute' => ['corners', 'highlight', 'icon', 'iconpos', 'iconshadow', 'inline', 'mini', 'shadow', 'theme', 'track-theme'],
+      'input-data-attribute' => ['clear-btn', 'clear-btn-text', 'corners', 'highlight', 'icon', 'iconpos', 'iconshadow', 'inline', 'mini', 'shadow', 'theme', 'track-theme'],
       'textarea-html-attribute' => ['id', 'name', 'class', 'rows', 'cols', 'readonly', 'disabled', 'title', 'required', 'placeholder', 'title', 'pattern'],
-      'textarea-data-attribute' => ['mini', 'theme'],
+      'textarea-data-attribute' => ['clear-btn', 'clear-btn-text', 'mini', 'theme'],
       'select-html-attribute' => ['id', 'class', 'size', 'maxlength', 'readonly', 'disabled', 'title', 'required', 'placeholder', 'title', 'pattern'],
       'select-data-attribute' => ['icon', 'iconpos', 'inline', 'mini', 'native-menu', 'overlay-theme', 'placeholder', 'theme'],
       'radio-checkbox-html-attribute' => ['id', 'class', 'readonly', 'disabled', 'title', 'required', 'placeholder', 'title', 'pattern', 'value'],
@@ -944,6 +1145,7 @@ C<form()> generates web forms. It accepts attributes defined in C<form-html-attr
       {type => 'radio', name => 'gender', options => ['Male', 'Female']},
       {type => 'checkbox', name => 'country', options => {'AU' => 'Austalia', 'US' => 'United States'}, value => 'AU'},
       {type => 'select', name => 'heard', label => 'How did you hear about us', options => ['Facebook', 'Twitter', 'Google', 'Radio', 'Other']},
+      {type => 'rangeslider', name => 'range', mini => 'true', from => {label => 'Range', name => 'from', min => 18, max => 100}, to => {name => 'to', min => 18, max => 100}},
     ],
     controlgroup => {type => 'horizontal'}, # use controlgroup to group the buttons, default to false, accepts "1" or a hashref
     buttons => [
@@ -1005,6 +1207,27 @@ C<listview()> generates listviews. It accepts attributes defined in C<listview-h
 
   print $jquery_mobile->page(content => $list);
 
+=head2 C<table>
+
+C<table()> generates tables. It accepts attributes defined in C<table-html-attribute> and C<table-data-attribute>.
+
+  my $table = $jquery_mobile->table(
+    class => 'ui-responsive',
+    th => {
+      'First Name' => {priority => '1'}, 
+      'Last Name' => {priority => '2'}, 
+      'Email' => {priority => '3'}, 
+      'Gender' => {priority => '4'}, 
+    },
+    headers => ['First Name', 'Last Name', 'Email', 'Gender'],
+    rows => [
+      ['John', 'Smith', 'john@work.com', 'Male'],
+      ['Ann', 'Smith', 'ann@work.com', 'Female'],
+    ],
+  );
+
+  print $jquery_mobile->page(content => $table);
+
 =over
 
 =item C<numbered>
@@ -1146,6 +1369,16 @@ C<button()> generates C<anchor> and C<input> buttons.
 
 Please note that C<anchor> buttons accepts C<button-html-anchor-attribute> as data-* attributes, whereas C<input> buttons uses C<button-data-attribute>. HTML attributes for both are defined in C<button-html-attribute>.
 
+=head2 C<panel>
+
+C<panel()> generates panel containers. Content can be passed via the C<content> parameter:
+
+  my $panel = $jquery_mobile->panel(
+  	content => 'Panel Content'
+  );
+
+It accepts attributes defined in C<panel-html-attribute> and C<panel-data-attribute>.
+
 =head2 C<controlgroup>
 
 C<controlgroup()> generates controlgroup containers. Content (usually buttons) can be passed via the C<content> parameter:
@@ -1213,6 +1446,19 @@ C<textarea()> generates textareas. It accepts attributes defined in C<textarea-h
     cols => '50'
   );
 
+=head2 C<rangeslider>
+
+C<rangeslider()> generates rangesliders. It accepts attributes defined in C<rangeslider-html-attribute> and C<rangeslider-data-attribute>.
+
+  print $jquery_mobile->rangeslider(
+    type => 'rangeslider', 
+    name => 'range', 
+    mini => 'true', 
+    from => {
+      label => 'Range', name => 'from', min => 18, max => 100}, to => {name => 'to', min => 18, max => 100}
+  );
+
+
 =head1 SEE ALSO
 
 L<http://jquerymobile.com>, L<https://github.com/dannyglue/jQuery-Mobilevalidate>
@@ -1223,7 +1469,7 @@ Xufeng (Danny) Liang (danny.glue@gmail.com)
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2012 Xufeng (Danny) Liang, All Rights Reserved.
+Copyright 2013 Xufeng (Danny) Liang, All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
